@@ -1,7 +1,7 @@
 // Copyright (c) Microsoft Corporation. All rights reserved.
 // Licensed under the MIT License.
 
-use anyhow::{bail, Context, Result};
+use crate::write_counter::Counter;
 use byteorder::{ByteOrder, LittleEndian, ReadBytesExt};
 use snap::write::FrameEncoder;
 use std::{
@@ -13,6 +13,38 @@ use std::{
     path::Path,
 };
 
+#[derive(thiserror::Error, Debug)]
+pub enum Error {
+    #[error("unable to write image")]
+    Write(#[source] std::io::Error),
+
+    #[error("unable to read memory")]
+    Read(#[source] std::io::Error),
+
+    #[error("unable to read header: {1}")]
+    ReadHeader(#[source] std::io::Error, &'static str),
+
+    #[error("invalid padding")]
+    InvalidPadding,
+
+    #[error("file is too large")]
+    TooLarge,
+
+    #[error("unimplemented version")]
+    UnimplementedVersion,
+
+    #[error("unsupported format")]
+    UnsupportedFormat,
+
+    #[error("write block failed: {0:?}")]
+    WriteBlock(Range<u64>),
+
+    #[error("size conversion error")]
+    SizeConversion,
+}
+
+type Result<T> = std::result::Result<T, Error>;
+
 pub const MAX_BLOCK_SIZE: u64 = 0x1000 * 0x1000;
 const PAGE_SIZE: usize = 0x1000;
 const LIME_MAGIC: u32 = 0x4c69_4d45; // EMiL as u32le
@@ -33,25 +65,25 @@ impl Header {
     pub fn read(mut src: &File) -> Result<Self> {
         let magic = src
             .read_u32::<LittleEndian>()
-            .context("unable to read magic")?;
+            .map_err(|e| Error::ReadHeader(e, "magic"))?;
         let version = src
             .read_u32::<LittleEndian>()
-            .context("unable to read version")?;
+            .map_err(|e| Error::ReadHeader(e, "version"))?;
         let start = src
             .read_u64::<LittleEndian>()
-            .context("unable to read start")?;
+            .map_err(|e| Error::ReadHeader(e, "start offset"))?;
         let end = src
             .read_u64::<LittleEndian>()
-            .context("unable to read end")?
+            .map_err(|e| Error::ReadHeader(e, "end offset"))?
             + 1;
         let padding = src
             .read_u64::<LittleEndian>()
-            .context("unable to read padding")?;
+            .map_err(|e| Error::ReadHeader(e, "padding"))?;
         if padding != 0 {
-            bail!("invalid padding: {}", padding);
+            return Err(Error::InvalidPadding);
         }
         if !(magic == LIME_MAGIC && version == 1 || magic == AVML_MAGIC && version == 2) {
-            bail!("unknown format");
+            return Err(Error::UnsupportedFormat);
         };
 
         Ok(Self {
@@ -64,7 +96,7 @@ impl Header {
         let magic = match self.version {
             1 => LIME_MAGIC,
             2 => AVML_MAGIC,
-            _ => bail!("unimplemented version"),
+            _ => return Err(Error::UnimplementedVersion),
         };
         let mut bytes = [0; 32];
         LittleEndian::write_u32_into(&[magic, self.version], &mut bytes[..8]);
@@ -77,7 +109,7 @@ impl Header {
         W: Write,
     {
         let bytes = self.encode()?;
-        dst.write_all(&bytes)?;
+        dst.write_all(&bytes).map_err(Error::Write)?;
         Ok(())
     }
 }
@@ -89,14 +121,14 @@ where
 {
     let mut buf = vec![0; PAGE_SIZE];
     while size >= PAGE_SIZE {
-        src.read_exact(&mut buf)?;
-        dst.write_all(&buf)?;
+        src.read_exact(&mut buf).map_err(Error::Read)?;
+        dst.write_all(&buf).map_err(Error::Write)?;
         size -= PAGE_SIZE;
     }
     if size > 0 {
         buf.resize(size, 0);
-        src.read_exact(&mut buf)?;
-        dst.write_all(&buf)?;
+        src.read_exact(&mut buf).map_err(Error::Read)?;
+        dst.write_all(&buf).map_err(Error::Write)?;
     }
     Ok(())
 }
@@ -105,10 +137,10 @@ where
 fn copy_if_nonzero<R, W>(header: &Header, src: &mut R, mut dst: &mut W) -> Result<()>
 where
     R: Read,
-    W: Write + Seek,
+    W: Write,
 {
     let size = usize::try_from(header.range.end - header.range.start)
-        .context("unable to create image range size")?;
+        .map_err(|_| Error::SizeConversion)?;
 
     // read the entire block into memory, but still read page by page
     let mut buf = Cursor::new(vec![0; size]);
@@ -122,20 +154,24 @@ where
 
     header.write(dst)?;
     if header.version == 1 {
-        dst.write_all(&buf)?;
+        dst.write_all(&buf).map_err(Error::Write)?;
     } else {
-        let begin = dst
-            .seek(SeekFrom::Current(0))
-            .context("unable to seek to location")?;
-        {
-            let mut snap_fh = FrameEncoder::new(&mut dst);
-            snap_fh.write_all(&buf)?;
-        }
-        let end = dst.seek(SeekFrom::Current(0)).context("seek failed")?;
+        let count = {
+            let mut counter = Counter::new(dst);
+            {
+                let mut snap_fh = FrameEncoder::new(&mut counter);
+                snap_fh.write_all(&buf).map_err(Error::Write)?;
+            }
+            let count = counter.count();
+            dst = counter.into_inner();
+            count
+        };
+        let count = count.try_into().map_err(|_| Error::SizeConversion)?;
+
         let mut size_bytes = [0; 8];
-        LittleEndian::write_u64_into(&[end - begin], &mut size_bytes);
-        dst.write_all(&size_bytes)
-            .context("write_all of size failed")?;
+        LittleEndian::write_u64_into(&[count], &mut size_bytes);
+
+        dst.write_all(&size_bytes).map_err(Error::Write)?;
     }
     Ok(())
 }
@@ -143,26 +179,30 @@ where
 fn copy_large_block<R, W>(header: &Header, src: &mut R, mut dst: &mut W) -> Result<()>
 where
     R: Read,
-    W: Write + Seek,
+    W: Write,
 {
     header.write(dst)?;
     let size = usize::try_from(header.range.end - header.range.start)
-        .context("unable to create image range size")?;
+        .map_err(|_| Error::SizeConversion)?;
+
     if header.version == 1 {
         copy(size, src, dst)?;
     } else {
-        let begin = dst
-            .seek(SeekFrom::Current(0))
-            .context("unable to seek to location")?;
-        {
-            let mut snap_fh = FrameEncoder::new(&mut dst);
-            copy(size, src, &mut snap_fh).context("copy failed")?;
-        }
-        let end = dst.seek(SeekFrom::Current(0)).context("seek failed")?;
+        let count = {
+            let mut counter = Counter::new(dst);
+            {
+                let mut snap_fh = FrameEncoder::new(&mut counter);
+                copy(size, src, &mut snap_fh)?;
+            }
+            let count = counter.count();
+            dst = counter.into_inner();
+            count
+        };
+        let count = count.try_into().map_err(|_| Error::SizeConversion)?;
+
         let mut size_bytes = [0; 8];
-        LittleEndian::write_u64_into(&[end - begin], &mut size_bytes);
-        dst.write_all(&size_bytes)
-            .context("write_all of size failed")?;
+        LittleEndian::write_u64_into(&[count], &mut size_bytes);
+        dst.write_all(&size_bytes).map_err(Error::Write)?;
     }
     Ok(())
 }
@@ -170,7 +210,7 @@ where
 fn copy_block_impl<R, W>(header: &Header, src: &mut R, dst: &mut W) -> Result<()>
 where
     R: Read,
-    W: Write + Seek,
+    W: Write,
 {
     if header.range.end - header.range.start > MAX_BLOCK_SIZE {
         copy_large_block(header, src, dst)
@@ -182,7 +222,7 @@ where
 pub fn copy_block<R, W>(mut header: Header, src: &mut R, dst: &mut W) -> Result<()>
 where
     R: Read,
-    W: Write + Seek,
+    W: Write,
 {
     if header.version == 2 {
         while header.range.end - header.range.start > MAX_BLOCK_SIZE {
@@ -197,14 +237,12 @@ where
                 },
                 src,
                 dst,
-            )
-            .with_context(|| format!("unable to copy block: {:?}", range))?;
+            )?;
             header.range.start += MAX_BLOCK_SIZE;
         }
     }
     if header.range.end > header.range.start {
-        copy_block_impl(&header, src, dst)
-            .with_context(|| format!("unable to copy block: {:?}", header.range))?;
+        copy_block_impl(&header, src, dst)?;
     }
 
     Ok(())
@@ -221,20 +259,22 @@ impl Image {
         let src = OpenOptions::new()
             .read(true)
             .open(src_filename)
-            .with_context(|| format!("unable to open source file: {}", src_filename.display()))?;
+            .map_err(Error::Read)?;
         let dst = OpenOptions::new()
             .mode(0o600)
             .write(true)
             .create(true)
             .truncate(true)
             .open(dst_filename)
-            .with_context(|| format!("unable to destination image: {}", dst_filename.display()))?;
+            .map_err(Error::Write)?;
+
         Ok(Self { version, src, dst })
     }
 
     pub fn write_blocks(&mut self, blocks: &[Block]) -> Result<()> {
         for block in blocks {
-            self.write_block(block)?;
+            self.write_block(block)
+                .map_err(|_| Error::WriteBlock(block.range.clone()))?;
         }
         Ok(())
     }
@@ -248,11 +288,10 @@ impl Image {
         if block.offset > 0 {
             self.src
                 .seek(SeekFrom::Start(block.offset))
-                .with_context(|| format!("unable to seek to block: {}", block.offset))?;
+                .map_err(Error::Read)?;
         }
 
-        copy_block(header, &mut self.src, &mut self.dst)
-            .with_context(|| format!("unable to copy block: {:?}", block.range))?;
+        copy_block(header, &mut self.src, &mut self.dst)?;
         Ok(())
     }
 }
